package org.gcube.gcat.workspace;

import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Map;
import java.util.concurrent.Callable;

import javax.ws.rs.InternalServerErrorException;

import org.gcube.common.gxhttp.request.GXHTTPStringRequest;
import org.gcube.common.security.AuthorizedTasks;
import org.gcube.common.security.secrets.Secret;
import org.gcube.common.storagehub.client.dsl.FileContainer;
import org.gcube.common.storagehub.model.Metadata;
import org.gcube.gcat.utils.Constants;
import org.gcube.storagehub.StorageHubManagement;
import org.glassfish.jersey.media.multipart.ContentDisposition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Luca Frosini (ISTI - CNR)
 */
public class CatalogueStorageHubManagement {
	
	private static final Logger logger = LoggerFactory.getLogger(CatalogueStorageHubManagement.class);
	
	protected StorageHubManagement storageHubManagement;
	
	protected String originalFilename;
	protected String mimeType;
	
	public String getOriginalFilename() {
		return originalFilename;
	}
	
	public String getMimeType() {
		return mimeType;
	}
	
	public CatalogueStorageHubManagement() {
		this.storageHubManagement = new StorageHubManagement();
	}
	
	protected String getOriginalFileName(HttpURLConnection httpURLConnection) throws Exception {
		String contentDisposition = httpURLConnection.getHeaderField("Content-Disposition");
		contentDisposition = contentDisposition.replaceAll("= ", "=").replaceAll(" =", "=");
		ContentDisposition formDataContentDisposition = new ContentDisposition(contentDisposition);
		return formDataContentDisposition.getFileName();
	}
	
	public URL ensureResourcePersistence(URL persistedURL, String itemID, String resourceID) throws Exception {
		Secret secret = Constants.getCatalogueSecret();
		try {
			
			Callable<URL> callable = new Callable<URL>() {

				@Override
				public URL call() throws Exception {
					GXHTTPStringRequest gxhttpStringRequest = GXHTTPStringRequest.newRequest(persistedURL.toString());
					gxhttpStringRequest.from(Constants.CATALOGUE_NAME);
					HttpURLConnection httpURLConnection = gxhttpStringRequest.get();
					mimeType = httpURLConnection.getContentType().split(";")[0];
					originalFilename = getOriginalFileName(httpURLConnection);
					CatalogueMetadata catalogueMetadata = new CatalogueMetadata(itemID);
					storageHubManagement.setCheckMetadata(catalogueMetadata);
					Metadata metadata = catalogueMetadata.getMetadata(persistedURL, originalFilename, resourceID);
					URL persistedURL = storageHubManagement.persistFile(httpURLConnection.getInputStream(), resourceID, mimeType,
							metadata);
					mimeType = storageHubManagement.getMimeType();
					return persistedURL;
				}
				
			};
			
			return AuthorizedTasks.executeSafely(callable, secret);
		} catch (Throwable t) {
			logger.error("Error while trying to persists the resource", t);
			throw new InternalServerErrorException(t);
		}
		
	}
	
	public void deleteResourcePersistence(String itemID, String resourceID, String mimeType) throws Exception {
		Secret secret = Constants.getCatalogueSecret();
		Runnable runnable = new Runnable() {
			@Override
			public void run() {
				try {
					storageHubManagement = new StorageHubManagement();
					CatalogueMetadata catalogueMetadata = new CatalogueMetadata(itemID);
					storageHubManagement.setCheckMetadata(catalogueMetadata);
					storageHubManagement.removePersistedFile(resourceID, mimeType);
				}catch (Exception e) {
					logger.error("Error while trying to delete resource with ID {} and mimetype {} of the item with ID {}", resourceID, mimeType, itemID, e);
					throw new InternalServerErrorException(e);
				}
			}
		};
		AuthorizedTasks.executeSafely(runnable, secret);
	}
	
	protected void internalAddRevisionID(String resourceID, String revisionID) throws Exception {
		FileContainer fileContainer = storageHubManagement.getPersistedFile();
		if(fileContainer != null) {
			Metadata metadata = fileContainer.get().getMetadata();
			Map<String,Object> map = metadata.getMap();
			map.put(CatalogueMetadata.CATALOGUE_RESOURCE_ID, resourceID);
			map.put(CatalogueMetadata.CATALOGUE_RESOURCE_REVISION_ID, revisionID);
			metadata.setMap(map);
			fileContainer.setMetadata(metadata);
		} else {
			logger.warn(
					"Unable to set revision id {} to the file of resource with id {} because the file was NOT found on storage-hub. This could be related to an item not created via gCat. Skipping it.",
					revisionID, resourceID);
		}
	}
	
	public void renameFile(String resourceID, String revisionID) throws Exception {
		Secret secret = Constants.getCatalogueSecret();
		Runnable runnable = new Runnable() {
			@Override
			public void run() {
				try {
					FileContainer createdfile = storageHubManagement.getPersistedFile();
					createdfile.rename(resourceID);
					internalAddRevisionID(resourceID, revisionID);
				}catch (Exception e) {
					logger.error("Error while trying to rename file of resource with ID {} and revision with ID {}", resourceID, revisionID, e);
					throw new InternalServerErrorException(e);
				}
			}
		};
		AuthorizedTasks.executeSafely(runnable, secret);
	}
	
	public void addRevisionID(String resourceID, String revisionID) throws Exception {
		Secret secret = Constants.getCatalogueSecret();
		Runnable runnable = new Runnable() {
			@Override
			public void run() {
				try {
					internalAddRevisionID(resourceID, revisionID);
				} catch (Exception e) {
					logger.error("Error while trying to add revision of resource with ID {} and revision with ID {}", resourceID, revisionID, e);
					throw new InternalServerErrorException(e);
				}
			}
		};
		AuthorizedTasks.executeSafely(runnable, secret);
	}
	
	public FileContainer retrievePersistedFile(String id, String mimeType) throws Exception {
		try {
			Secret secret = Constants.getCatalogueSecret();
			Callable<FileContainer> callable = new Callable<FileContainer>() {
				@Override
				public FileContainer call() throws Exception {
					try {
						return storageHubManagement.getPersistedFile(id, mimeType);
					} catch (Exception e) {
						logger.error("Error while trying to read file with ID {} and mimetype {} from storagehub", id, mimeType, e);
						throw e;
					}
				}
			};
			return AuthorizedTasks.executeSafely(callable, secret);
		} catch (Throwable t) {
			logger.error("Error while trying to read file with ID {} and mimetype {} from storagehub", id, mimeType, t);
			throw new InternalServerErrorException(t);
		}
	}
	
	public FileContainer getPersistedFile() {
		return storageHubManagement.getPersistedFile();
	}
	
}
